# Copyright 2017 David R. Bild
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License

from __future__ import absolute_import

import ssl
import weakref

import _ssl

if ssl.OPENSSL_VERSION_INFO >= (3, 0):
    from sslpsk_pmd3 import _sslpsk_openssl3 as _sslpsk
else:
    from sslpsk_pmd3 import _sslpsk_openssl1 as _sslpsk


_callbacks = {}


class FinalizerRef(weakref.ref):
    """subclass weakref.ref so that attributes can be added"""


def _register_callback(sock, ssl_id, callback):
    _callbacks[ssl_id] = callback
    callback.unregister = FinalizerRef(sock, _unregister_callback)
    callback.unregister.ssl_id = ssl_id


def _unregister_callback(ref):
    del _callbacks[ref.ssl_id]


def _python_psk_client_callback(ssl_id, hint):
    """Called by _sslpsk.c to return the (psk, identity) tuple for the socket with
    the specified ssl socket.

    """
    if ssl_id not in _callbacks:
        return (b"", b"")
    else:
        res = _callbacks[ssl_id](hint)
        return res if isinstance(res, tuple) else (res, b"")


def _sslobj(sock):
    """Returns the underlying PySLLSocket object with which the C extension
    functions interface.

    """
    if isinstance(sock._sslobj, _ssl._SSLSocket):
        return sock._sslobj
    else:
        return sock._sslobj._sslobj


def _python_psk_server_callback(ssl_id, identity):
    """Called by _sslpsk.c to return the psk for the socket with the specified
    ssl socket.

    """
    if ssl_id not in _callbacks:
        return b""
    else:
        return _callbacks[ssl_id](identity)


_sslpsk.sslpsk_set_python_psk_client_callback(_python_psk_client_callback)
_sslpsk.sslpsk_set_python_psk_server_callback(_python_psk_server_callback)


def _ssl_set_psk_client_callback(sock, psk_cb):
    ssl_id = _sslpsk.sslpsk_set_psk_client_callback(_sslobj(sock))
    _register_callback(sock, ssl_id, psk_cb)


def _ssl_set_psk_server_callback(sock, psk_cb, hint):
    ssl_id = _sslpsk.sslpsk_set_accept_state(_sslobj(sock))
    _ = _sslpsk.sslpsk_set_psk_server_callback(_sslobj(sock))
    _ = _sslpsk.sslpsk_use_psk_identity_hint(_sslobj(sock), hint if hint else b"")
    _register_callback(sock, ssl_id, psk_cb)

def _ssl_setup_psk_callbacks(sslobj):
    psk = sslobj.context.psk
    hint = sslobj.context.hint
    if psk:
        if sslobj.server_side:
            cb = psk if callable(psk) else lambda _identity: psk
            _ssl_set_psk_server_callback(sslobj, cb, hint)
        else:
            cb = psk if callable(psk) else lambda _hint: psk if isinstance(psk, tuple) else (psk, b"")
            _ssl_set_psk_client_callback(sslobj, cb)


class SSLPSKContext(ssl.SSLContext):
    @property
    def psk(self):
        return getattr(self, "_psk", None)

    @psk.setter
    def psk(self, psk):
        self._psk = psk

    @property
    def hint(self):
        return getattr(self, "_hint", None)

    @hint.setter
    def hint(self, hint):
        self._hint = hint


class SSLPSKObject(ssl.SSLObject):
    def do_handshake(self, *args, **kwargs):
        _ssl_setup_psk_callbacks(self)
        super().do_handshake(*args, **kwargs)


class SSLPSKSocket(ssl.SSLSocket):
    def do_handshake(self, *args, **kwargs):
        _ssl_setup_psk_callbacks(self)
        super().do_handshake(*args, **kwargs)


SSLPSKContext.sslobject_class = SSLPSKObject
SSLPSKContext.sslsocket_class = SSLPSKSocket

def wrap_socket(*args, **kwargs):
    """ """
    do_handshake_on_connect = kwargs.get("do_handshake_on_connect", True)
    kwargs["do_handshake_on_connect"] = False

    psk = kwargs.setdefault("psk", None)
    del kwargs["psk"]

    hint = kwargs.setdefault("hint", None)
    del kwargs["hint"]

    server_side = kwargs.setdefault("server_side", False)
    if psk:
        del kwargs["server_side"]  # bypass need for cert

    sock = ssl.wrap_socket(*args, **kwargs)

    if psk:
        if server_side:
            cb = psk if callable(psk) else lambda _identity: psk
            _ssl_set_psk_server_callback(sock, cb, hint)
        else:
            cb = (
                psk
                if callable(psk)
                else lambda _hint: psk
                if isinstance(psk, tuple)
                else (psk, b"")
            )
            _ssl_set_psk_client_callback(sock, cb)

    if do_handshake_on_connect:
        sock.do_handshake()

    return sock



# Copyright 2017 David R. Bild
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License

import os
import socket
import ssl
import sys
import threading
import unittest

import sslpsk_pmd3

HOST='localhost'
PORT=6000
TEST_DATA=b'abcdefghi'

class sslpsk3Test(unittest.TestCase):
    # ---------- setup/tear down functions
    def setUp(self):
        self.psk = b'c033f52671c61c8128f7f8a40be88038bcf2b07a6eb3095c36e3759f0cf40837'
        self.addr = (HOST, PORT)
        self.client_socket = socket.socket()
        self.server_socket = None
        self.accept_socket = socket.socket()
        self.client_psk_sock = None
        self.server_psk_sock = None

        self.startServer()
    
    def tearDown(self):
        for sock in [self.client_psk_sock or self.client_socket,
                     self.server_psk_sock or self.server_socket,
                     self.accept_socket]:
            try:
                sock.shutdown(socket.SHUT_RDWR)
            except socket.error:
                pass
            finally:
                sock.close()

        self.client_socket = None
        self.server_socket = None
        self.accept_socket = None
        self.client_psk_sock = None
        self.server_psk_sock = None

    def startServer(self):
        self.accept_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.accept_socket.bind(self.addr)
        self.accept_socket.listen(1)

        def accept():
            self.server_socket, _ = self.accept_socket.accept()
        
            # wrap socket with TLS-PSK
            self.server_psk_sock = sslpsk_pmd3.wrap_socket(self.server_socket, psk=self.psk, ciphers='PSK-AES256-CBC-SHA',
                                                           ssl_version=ssl.PROTOCOL_TLSv1, server_side=True)
        
            # accept data from client
            data = self.server_psk_sock.recv(10)
            self.server_psk_sock.sendall(data.upper())

        threading.Thread(target = accept).start()

    def testClient(self):
        # initialize
        self.client_socket.connect(self.addr)
        
        # wrap socket with TLS-PSK
        self.client_psk_sock = sslpsk_pmd3.wrap_socket(self.client_socket, psk=self.psk, ciphers='PSK-AES256-CBC-SHA',
                                                       ssl_version=ssl.PROTOCOL_TLSv1, server_side=False)
        
        self.client_psk_sock.sendall(TEST_DATA)
        data = self.client_psk_sock.recv(10)
        print('data: %s' % data)
        self.assertTrue(data == TEST_DATA.upper(), 'Test Failed')

def main():
    unittest.main(buffer=False)

if __name__ == '__main__':
    main()



from __future__ import print_function

import socket
import ssl

import sslpsk_pmd3

PSKS = {'server1' : b'abcdef',
        'server2' : b'uvwxyz'}

def client(host, port, psk):
    tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    tcp_socket.connect((host, port))

    ssl_sock = sslpsk_pmd3.wrap_socket(tcp_socket,
                                       ssl_version=ssl.PROTOCOL_TLSv1,
                                       ciphers='ALL:!ADH:!LOW:!EXP:!MD5:@STRENGTH',
                                       psk=lambda hint: (PSKS[hint], b'client1'))

    msg = "ping"
    ssl_sock.sendall(msg.encode())
    msg = ssl_sock.recv(4).decode()
    print('Client received: %s'%(msg))

    ssl_sock.shutdown(socket.SHUT_RDWR)
    ssl_sock.close()

def main():
    host = '127.0.0.1'
    port = 6000
    client(host, port, PSKS)

if __name__ == '__main__':
    main()



# Copyright 2017 David R. Bild
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License

import sys

from setuptools import Extension, setup

if sys.platform == 'win32':
    libraries = ['libcrypto', 'libssl']
else:
    libraries = ['crypto', 'ssl']

extension_openssl1 = Extension(
    "sslpsk_pmd3._sslpsk_openssl1",
    sources=["sslpsk_pmd3/_sslpsk.c"],
    libraries=libraries,
    include_dirs=["openssl1/include/"],
    library_dirs=["openssl1/lib/VC/"],
    define_macros=[
        ("OPENSSL_VER", "openssl1"),
        ("INIT_SSLPSK_OPENSSL", "init_sslpsk_openssl1"),
        ("PYINIT_SSLPSK_OPENSSL", "PyInit__sslpsk_openssl1"),
    ],
)

extension_openssl3 = Extension(
    "sslpsk_pmd3._sslpsk_openssl3",
    sources=["sslpsk_pmd3/_sslpsk.c"],
    libraries=libraries,
    include_dirs=["openssl3/include/"],
    library_dirs=["openssl3/lib/VC/"],
    define_macros=[
        ("OPENSSL_VER", "openssl3"),
        ("INIT_SSLPSK_OPENSSL", "init_sslpsk_openssl3"),
        ("PYINIT_SSLPSK_OPENSSL", "PyInit__sslpsk_openssl3"),
    ],
)

setup(name='pytun-pmd3', ext_modules=[extension_openssl1, extension_openssl3])



from __future__ import print_function

import socket
import ssl

import sslpsk_pmd3

PSKS = {'client1' : b'abcdef',
        'client2' : b'123456'}

def server(host, port):
    tcp_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    tcp_sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    tcp_sock.bind((host, port))
    tcp_sock.listen(1)

    sock, _ = tcp_sock.accept()
    ssl_sock = sslpsk_pmd3.wrap_socket(sock,
                                       server_side = True,
                                       ssl_version=ssl.PROTOCOL_TLSv1,
                                       ciphers='ALL:!ADH:!LOW:!EXP:!MD5:@STRENGTH',
                                       psk=lambda identity: PSKS[identity],
                                       hint=b'server1')

    msg = ssl_sock.recv(4).decode()
    print('Server received: %s'%(msg))
    msg = "pong"
    ssl_sock.sendall(msg.encode())

    ssl_sock.shutdown(socket.SHUT_RDWR)
    ssl_sock.close()

def main():
    host = '127.0.0.1'
    port = 6000
    server(host, port)

if __name__ == '__main__':
    main()
